//
//  Attaques.swift
//  Swiftement Pokemon
//
//  Created by Onoulade on 15/01/2022.
//

import Foundation

/*
 La classe attaque qui se conforme au protocole de capacité
 */

class Attaque: Capacite {
    
    var nom: String
    var element: Element?
    var pointsDegats: Float
    var recharges: Int
    var rechargesMax: Int
    
    // Chaque attaque possède un nom, des points de dégats infligés et un nombre d'utilisation
    // L'élément est optionnel
    init(nom: String, element: Element? = nil, pointsDegats: Float, recharges: Int) {
        self.nom = nom
        self.element = element
        self.pointsDegats = pointsDegats
        self.recharges = recharges
        self.rechargesMax = recharges
    }
    
    // On appellera cette fonction pour utiliser l'attaque d'un pokémon sur un autre
    func utiliser(avec joueur: Pokemon, sur ennemi: Pokemon) -> Bool {
        if recharges == 0 {
            print("L'attaque ", nom, " n'a plus de recharges !")
            return false
        }
        
        print(joueur.nom, "attaque", ennemi.nom, "avec", nom)
        
        // On multiplie éventuellement les points de dégats en fonction de l'efficacité des éléments
        
        var multiplicateur: Float = 1
        if let element = self.element {
            switch(Element.efficace(element, sur: ennemi.element)) {
            case true:
                multiplicateur = Element.multipleEfficacite
                print("C'est très efficace !")
            case false:
                multiplicateur = Element.diviseurEfficacite
                print("C'est pathétique !")
            default:
                multiplicateur = 1
            }
        }
        
        // On inflige les points de dégats
        let totalDegats = pointsDegats * multiplicateur
        ennemi.subit(pointsDegats: totalDegats)
        
        // Et on oublie pas de décompter une utilisation
        self.recharges -= 1
        return true
    }
    
    // Initialisation de toutes les attaques
    
    static var coupDeBoule: Attaque {
        get {
            return .init(nom: "Coup de boule", pointsDegats: 7, recharges: 4)
        }
    }
    static var bouleDeFeu: Attaque {
        get {
            return .init(nom: "Boule de feu", pointsDegats: 5, recharges: 5)
        }
    }
    static var gravillons: Attaque {
        get {
            return .init(nom: "Gravillons", element: .pierre, pointsDegats: 5, recharges: 7)
        }
    }
    static var rosier: Attaque  {
        get {
            return .init(nom: "Rosier", element: .feuille, pointsDegats: 6, recharges: 4)
        }
    }
    static var couteaux: Attaque {
        get {
            return .init(nom: "Couteaux", element: .ciseaux, pointsDegats: 8, recharges: 2)
        }
    }
}
